@@ -237,7 +237,7 @@ bootstrap_remote_freebsd()
     # make additional wheels available for packages which lack them for this platform
     echo "# generated by ansible-test
 [global]
-extra-index-url = https://spare-tire.testing.ansible.com/simple/
+extra-index-url = https://spare-tire.core.ansible.com/simple/
 prefer-binary = yes
 " > /etc/pip.conf
 
